/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Unit test about the content of the distribution archive
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PackageIT {

    private final static String PROPERTY_NAME_ARCHIVE_FILE = "archive-file";

    private static File ARCHIVE_FILE = null;
    
    private static String EXPECTED_ENTRIES[] = {
            // Samples for Enterprise Edition
            "sa-camel-datamapping",
            "sa-camel-hello-proxies",
            "sa-soap-datamapping",
            "sa-soap-hello-proxies",
            "sa-eip",
            "sa-filetransfer",
            "sa-ftp",
            "sa-jsr181-collection",
            "sa-jsr181-hello",
            "sa-mail",
            "sa-quartz",
            "sa-sftp",
            "sa-sql",
            "sa-activiti-vacation-sample",
            "sa-flowable-vacation-sample",
            "sa-validation",
            "sa-xslt",
            "sa-xslt-autogenerate",
            "sa-gateway-hello-domain0",
            "sa-gateway-hello-domain1",
            
            // Samples for Community Pack
            "sa-jms",
            "sa-pojo",
            "sa-rest",
            "sa-talend-ArticleImporterJobService-provide",
            "sa-talend-sample",

            "LICENSE",
            "THIRD-PARTY",
            "COPYRIGHT"
    };

    @BeforeClass
    public static void setup() {
        final String archiveBaseDirectoryStr = System.getProperty(PROPERTY_NAME_ARCHIVE_FILE);
        assertNotNull("System property '" + PROPERTY_NAME_ARCHIVE_FILE + "' not set.",
                archiveBaseDirectoryStr);
        ARCHIVE_FILE = new File(archiveBaseDirectoryStr);
        assertTrue("The archive file '" + ARCHIVE_FILE.getAbsolutePath() + "' does not exist", ARCHIVE_FILE.exists());
        assertTrue("The archive file '" + ARCHIVE_FILE.getAbsolutePath() + "' is not a file", ARCHIVE_FILE.isFile());
    }

    /**
     * <p>
     * Verify the content of the archive.
     * </p>
     */
    @Test
    public void content() throws IOException {
        
        final InputStream inArchive = new FileInputStream(ARCHIVE_FILE);
        assertNotNull(inArchive);
        try {
            int zipEntryCount = 0;
            final ZipInputStream zipArchive = new ZipInputStream(inArchive);
            ZipEntry zipEntry;
            while ((zipEntry = zipArchive.getNextEntry()) != null) {
                if (!zipEntry.isDirectory()) {
                    final String zipEntryName = zipEntry.getName();
                    boolean zipEntryOk = false;
                    for (final String expectedEntry : EXPECTED_ENTRIES) {
                        if (zipEntry.getName().contains(expectedEntry)) {
                            zipEntryOk = true;
                        }
                    }
                    assertTrue("Unexpected content: " + zipEntryName, zipEntryOk);
                    zipEntryCount++;
                }
            }
            assertEquals("Unexpected number of files in archive", EXPECTED_ENTRIES.length, zipEntryCount);
        } finally {
            inArchive.close();
        }
    }
}
